#import os
#thisdir = os.path.dirname(__file__)
#thisdir = os.path.abspath(thisdir)

#pythondir = os.path.join(thisdir, '..')
#pythondir = os.path.abspath(pythondir)

#import sys
#sys.path.insert(0, pythondir)



#! /usr/bin/env python


import pytest


from rec_env import load_configuration


TEST_YAML = '''
carrier: mycarrier
environment: uat
url: "{carrier}.{hosts[{environment}]}"
features:
  contacts: true
  backup: False
hosts:
  production: "{environment}.urlprod.com"
  uat: "{environment}.urltest.net"
  dev: localhost
clients:
  ios:
     provision: "{url}"
     auth: "{hosts[{environment}]}/auth"
api_version: 1
'''

CIRCULAR_YAML = '''
carrier: "{circular}"
circular: "{carrier}"
'''

def test_recursive_replace():
    env = load_configuration(TEST_YAML)
    assert env['url'] == "mycarrier.uat.urltest.net"


def test_recursive_replace_with_keywords():
    env = load_configuration(TEST_YAML, {'environment': 'dev'})
    assert env['url'] == "mycarrier.localhost"


def test_recursive_replace_boolean_values():
    env = load_configuration(TEST_YAML)
    assert env['features']['contacts'] is True
    assert env['features']['backup'] is False


def test_recursive_replace_nested_simple():
    env = load_configuration(TEST_YAML)
    assert env['clients']['ios']['provision'] == "mycarrier.uat.urltest.net"


def test_recursive_replace_nested_complex():
    env = load_configuration(TEST_YAML)
    assert env['clients']['ios']['auth'] == "uat.urltest.net/auth"


def test_int_values():
    env = load_configuration(TEST_YAML)
    assert env['api_version'] == 1


def test_recursive_circular_variables():
    # Notice: if this fails we probably made an AI or a parallel universe.
    with pytest.raises(RuntimeError):
        env = load_configuration(CIRCULAR_YAML)



#!/usr/bin/env python

from __future__ import print_function, division
import re

import yaml


def load_configuration(yaml_string, kw=None):
    environ = yaml.safe_load(yaml_string)
    environ = _expand_config_vars(environ, updates=kw)

    return environ


def _rec_replace(env, value):
    finder_subkeys = re.compile('\[\{(\w*)\}\]')
    finder_vars = re.compile('\{(\w*)\}')
    newvalue = value

    for k in finder_subkeys.findall(value):
        newvalue = _rec_replace(env, newvalue.replace("{%s}" % k, env[k]))

    newvalue = newvalue.format(**env)
    keys = finder_vars.findall(newvalue)
    while keys:
        for k in keys:
            newvalue = _rec_replace(env, newvalue.replace("{%s}" % k, env[k]))
        keys = finder_vars.findall(newvalue)

    return newvalue


def _env_replace(old_env, ref=None):
    new_env = {}
    for k in old_env:
        try:
            # tests if value is a string
            old_env[k].split(' ')
        except AttributeError:
            # if it's not a string, let's test if it is a dict
            try:
                old_env[k].keys()
            except AttributeError:
                # if it's not, just set new_env with it
                new_env[k] = old_env[k]
            else:
                # Yup, a dict. Need to replace recursively too.
                new_env[k] = _env_replace(old_env[k],
                                          ref if ref else old_env)
        else:
                # else start replacing vars
                new_env[k] = _rec_replace(ref if ref else old_env,
                                          old_env[k])

    return new_env


def _expand_config_vars(d, updates=None):

    if updates:
        d.update(updates)
    new_d = _env_replace(d)

    return new_d



from setuptools import setup, find_packages, Command
import os

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()
NEWS = open(os.path.join(here, 'NEWS.txt')).read()

version = "0.0.2"

install_requires = [
    'PyYAML'
]


class PyTest(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import sys
        import subprocess
        errno = subprocess.call([sys.executable, 'runtests.py'])
        raise SystemExit(errno)


setup(name='rec_env',
      version=version,
      description="",
      long_description=README + '\n\n' + NEWS,
      classifiers=[
      # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      ],
      author='Luiz Irber',
      author_email='luiz.irber@gmail.com',
      license='PSF',
      packages=find_packages(),
      #package_dir={'': 'rec_env'},
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      cmdclass={'test': PyTest},
      platforms='any',
      )
